#define GxEPD2_DISPLAY_CLASS GxEPD2_BW
#define GxEPD2_DRIVER_CLASS GxEPD2_420
#ifndef EPD_CS
#define EPD_CS SS
#endif
#define GxEPD2_BW_IS_GxEPD2_BW true

#include <Adafruit_GFX.h>

#include <Fonts/FreeSansBold24pt7b.h>
#include <Fonts/FreeSansBold18pt7b.h>
#include <Fonts/FreeSansBold12pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeSans9pt7b.h>

#include <GxEPD2_BW.h>
#include <Wire.h>
#include <WiFi.h>
#include<EEPROM.h>
#include "HomeConsole.h"
#include "assets.h"

// EPAPER INIT
GxEPD2_BW<GxEPD2_420, GxEPD2_420::HEIGHT> display(GxEPD2_420(/*CS=D8*/ SS, /*DC=D3*/ 17, /*RST=D4*/ 16, /*BUSY=D2*/ 4));

// TEMP & HUMIDITY SENSOR
#define Addr 0x40

// WIFI
const char* ssid     = "";
const char* password = "";

// API CALLS
HomeConsole rpi;

// WAKEUP BUTTONS
//001000000000000
//010000000000000
//100000000000000
//111000000000000
#define BUTTON_PIN_BITMASK 0x7000

void drawDashbaord(int reason);

struct EventDate {
  String milk;
  String plant;
};

void writeEvent(EventDate data) {
  EEPROM.put(0, data);
  EEPROM.commit();
  delay(50);
}

EventDate readEvent() {
  EventDate data;
  EEPROM.get(0, data);
  return data;
}

int print_GPIO_wake_up() {
  int GPIO_reason = esp_sleep_get_ext1_wakeup_status();
  Serial.print("GPIO that triggered the wake up: GPIO ");
  Serial.println((log(GPIO_reason)) / log(2), 0);
  return int(log(GPIO_reason) / log(2));
}


void drawLoading() {
  display.setRotation(0);
  display.setTextColor(GxEPD_BLACK);
  char* loading = "Updating";
  display.setFont(&FreeSans9pt7b);
  int16_t tbx, tby; uint16_t tbw, tbh;
  display.getTextBounds(loading, 0, 0, &tbx, &tby, &tbw, &tbh);
  display.firstPage();
  do
  {
    display.fillScreen(GxEPD_WHITE);
    display.setCursor(200 - tbw / 2, 150 - tbh / 2);
    display.print(loading);
  }
  while (display.nextPage());
}

void setup() {
  display.init();
  drawLoading();

  Wire.begin();
  EEPROM.begin(512);
  Serial.begin(115200);

  // WIFI SETUP
  WiFi.persistent(true);
  WiFi.mode(WIFI_STA);
  WiFi.setAutoConnect(true);
  WiFi.setAutoReconnect(true);
  WiFi.begin(ssid, password);
  int retries = 0;
  while (WiFi.status() != WL_CONNECTED) {
    retries++;
    delay(100);
    Serial.print(".");
    if (retries > 100) {
      break;
    }
  }

  /**
   13 - Reload
   12 - Milk
   14 - Plant
  */
  int reason = print_GPIO_wake_up();
  drawDashbaord(reason);

  // PREPARE FOR SLEEP
  esp_sleep_enable_ext1_wakeup(BUTTON_PIN_BITMASK, ESP_EXT1_WAKEUP_ANY_HIGH);

  //Go to sleep now
  Serial.println("Going to sleep now");
  delay(1000);
  esp_sleep_enable_timer_wakeup(1ULL * 60 * 60 * 1000 * 1000);
  esp_deep_sleep_start();
  Serial.println("Sleep");
}

void loop() {
}

const unsigned char* stringToIcon(String iconName, bool isSmall) {
  if (iconName == "02") {
    return isSmall ? sd02 : d02;
  }
  if (iconName == "03" || iconName == "04") {
    return isSmall ? sd03 : d03;
  }
  if (iconName == "09" || iconName == "10" ) {
    return isSmall ? sd09 : d09;
  }
  if (iconName == "11") {
    return isSmall ? sd11 : d11;
  }
  if (iconName == "13") {
    return isSmall ? sd13 : d13;
  }
  if (iconName == "50") {
    return isSmall ? sd50 : d50;
  }
  return isSmall ? sd02 : d02;
}

void getSensorData(int &rtemp, int &rhumidity) {
  unsigned int data[2];

  // Start I2C transmission
  Wire.beginTransmission(Addr);
  // Send humidity measurement command, NO HOLD MASTER
  Wire.write(0xF5);
  Wire.endTransmission();
  delay(200);

  Wire.requestFrom(Addr, 2);

  if (Wire.available() == 2) {
    data[0] = Wire.read();
    data[1] = Wire.read();
  }

  // Convert the data
  float humidity  = ((data[0] * 256.0) + data[1]);
  humidity = ((125 * humidity) / 65536.0) - 6;

  Wire.beginTransmission(Addr);
  // Send temperature measurement command, NO HOLD MASTER
  Wire.write(0xF3);
  Wire.endTransmission();
  delay(200);

  Wire.requestFrom(Addr, 2);

  if (Wire.available() == 2) {
    data[0] = Wire.read();
    data[1] = Wire.read();
  }

  // Convert the data
  float temp  = ((data[0] * 256.0) + data[1]);
  float ctemp = ((175.72 * temp) / 65536.0) - 46.85;
  rtemp = int(ctemp);
  rhumidity = int(humidity);
}

void drawDashbaord(int reason) {
  display.setTextColor(GxEPD_BLACK);

  // GET API data
  const String parseStatus = rpi.parseData();
  Serial.print("Parse status: ");
  Serial.println(parseStatus);
  if (parseStatus != "OK") {
    return;
  }
  Serial.println(rpi.date);

  EventDate stuff = readEvent();
  stuff.milk = stuff.milk.length() == 5 ? stuff.milk : "--.--";
  stuff.plant = stuff.plant.length() == 5 ? stuff.plant : "--.--";

  if (reason == 12 || reason == 14) {
    if (reason == 14) {
      stuff.plant = rpi.clickTime;
    } else {
      stuff.milk = rpi.clickTime;
    }
    writeEvent(stuff);
  }
  Serial.println(reason);

  display.firstPage();
  do {
    display.fillScreen(GxEPD_WHITE);

    // DAHBOARD
    display.drawInvertedBitmap(0, 0, dashboard, 400, 300, GxEPD_BLACK);

    // MENU BUTTONS
    display.setFont(&FreeSans9pt7b);
    display.setTextColor(GxEPD_BLACK);
    display.setCursor(14, 22);
    display.print("Reload");

    display.setCursor(105, 22);
    display.print("Milk");

    display.setCursor(180, 22);
    display.print("Plant");


    // SENSOR DATA
    int t;
    int h;
    getSensorData(t, h);
    Serial.println(t);
    Serial.println(h);
    String humidity = String(h) + "%";
    String temp = String(t) + "C";

    int16_t tbx, tby; uint16_t tbw, tbh;
    display.setFont(&FreeSansBold18pt7b);
    display.getTextBounds(humidity, 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(180 - tbw / 2, 75);
    display.print(temp);

    display.setCursor(180 - tbw / 2, 115);
    display.print(humidity);

    // MEMORY DATA
    String milkDate = stuff.milk;
    String plantDate = stuff.plant;
    display.setFont(&FreeSans12pt7b);

    // MILK
    display.getTextBounds(milkDate, 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(60 - tbw / 2, 220);
    display.print(milkDate);

    // PLANT
    display.getTextBounds(plantDate, 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(180 - tbw / 2, 220);
    display.print(plantDate);

    // EVENTS
    display.setFont(&FreeSans9pt7b);
    display.setTextColor(GxEPD_WHITE);
    display.setCursor(14, 255);
    display.print(String(rpi.event_0).substring(0,24));

    display.setCursor(14, 285);
    display.print(String(rpi.event_1).substring(0,24));
    

    // DATE
    String date(rpi.date);
    Serial.println(date);
    String month = date.substring(4, 7);
    String day = date.substring(8, 10);
    display.setFont(&FreeSansBold24pt7b);

    // DAY
    display.getTextBounds(day, 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(320 - tbw / 2, 50);
    display.print(day);

    // MONTH
    display.getTextBounds(month, 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(320 - tbw / 2, 110);
    display.print(month);

    // WAETHER
    display.drawInvertedBitmap(274, 135, stringToIcon(rpi.weather_icon, false), 92, 92, GxEPD_WHITE);
    display.setFont(&FreeSansBold18pt7b);
    display.getTextBounds(String(rpi.weather_temp), 0, 0, &tbx, &tby, &tbw, &tbh);
    display.setCursor(320 - tbw / 2, 264);
    display.print(String(rpi.weather_temp));
  }
  while (display.nextPage());
}
